% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( Matrix )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [malate_Matrix , NAD_Matrix , oxaloacetate_Matrix , NADH_Matrix , ATP_Matrix , ADP_Matrix , AMP_Matrix ]
%
%
% Free Parameters:
% [Vmf_MDH ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
%% LIST OF STATE VARIABLES
% 1 malate_Matrix
% 2 NAD_Matrix
% 3 oxaloacetate_Matrix
% 4 NADH_Matrix
% 5 ATP_Matrix
% 6 ADP_Matrix
% 7 AMP_Matrix
% 8 h_Matrix
% 9 m_Matrix
% 10 k_Matrix

% PARTIAL VOLUME FRACTIONS
VWater_Matrix = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_Matrix = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
malate_Matrix = x(1);
NAD_Matrix = x(2);
oxaloacetate_Matrix = x(3);
NADH_Matrix = x(4);
ATP_Matrix = x(5);
ADP_Matrix = x(6);
AMP_Matrix = x(7);
% Concentrations of H, Mg, and K
h_Matrix = x(8);
m_Matrix = x(9);
k_Matrix = x(10);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% malate_Matrix
Kh(1) = 2.3094527490892961726085333418190259635593974962831e-05;
Km(1) = 0.027990983755242043046207456313823058735579252243042;
Kk(1) = 0.79160600693895299961866385274333879351615905761719;
% NAD_Matrix
Kh(2) = Inf;
Km(2) = Inf;
Kk(2) = Inf;
% oxaloacetate_Matrix
Kh(3) = 0.00015083741503259175554402249375129940744955092668533;
Km(3) = 0.13709390976603802458910763562016654759645462036133;
Kk(3) = Inf;
% NADH_Matrix
Kh(4) = Inf;
Km(4) = Inf;
Kk(4) = Inf;
% ATP_Matrix
Kh(5) = 2.7990983755242071437057782383461823627612830023281e-07;
Km(5) = 0.00010815244062499600717370401570249782707833219319582;
Kk(5) = 0.097055055483606045285682739631738513708114624023438;
% ADP_Matrix
Kh(6) = 4.1856568564882886504828268456335216285424394300207e-07;
Km(6) = 0.0008821191357550335187392986924237447965424507856369;
Kk(6) = 0.13114858875318427533507303905935259535908699035645;
% AMP_Matrix
Kh(7) = 6.4641151032791652445184471878492082907996518770233e-07;
Km(7) = 0.021233332579732779737957670818104816135019063949585;
Kk(7) = 0.23906119225358615776322324109059991315007209777832;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_Matrix/Kh(1) + m_Matrix/Km(1) + k_Matrix/Kk(1);
P( 2 ) = 1  + h_Matrix/Kh(2) + m_Matrix/Km(2) + k_Matrix/Kk(2);
P( 3 ) = 1  + h_Matrix/Kh(3) + m_Matrix/Km(3) + k_Matrix/Kk(3);
P( 4 ) = 1  + h_Matrix/Kh(4) + m_Matrix/Km(4) + k_Matrix/Kk(4);
P( 5 ) = 1  + h_Matrix/Kh(5) + m_Matrix/Km(5) + k_Matrix/Kk(5);
P( 6 ) = 1  + h_Matrix/Kh(6) + m_Matrix/Km(6) + k_Matrix/Kk(6);
P( 7 ) = 1  + h_Matrix/Kh(7) + m_Matrix/Km(7) + k_Matrix/Kk(7);

%% THERMODYNAMIC EQUATIONS
DGro_MDH =69.1235;

Keq_MDH_Matrix = exp(-DGro_MDH/RT)/P(1)/P(2)*P(3)*P(4)/h_Matrix;

%% FLUX EQUATIONS
%MDH_Matrix
a=NAD_Matrix;
b=malate_Matrix;
p=oxaloacetate_Matrix;
q=NADH_Matrix;
KmA=90.55e-6;
KmB=250e-6;
KmP=6.128e-6;
KmQ=2.58e-6;
Kia=279e-6;
Kib=360e-6;
Kip=5.5e-6;
Kiq=3.18e-6;
KiATP=183.2e-6;
KiADP=394.4e-6;
KiAMP=420.0e-6;
fATP=ATP_Matrix*(1+h_Matrix/Kh(5))/P(5);
fADP=ADP_Matrix*(1+h_Matrix/Kh(6))/P(6);
fAMP=AMP_Matrix*(1+h_Matrix/Kh(7))/P(7);
ai=1+fATP/KiATP+fADP/KiADP+fAMP/KiAMP;
Vmf_MDH=par(1);
Vmr=Vmf_MDH/Keq_MDH_Matrix*(Kiq*KmP/Kia/KmB);
J_MDH_Matrix=(Vmf_MDH*Vmr*a*b-Vmf_MDH*Vmr*(p*q/Keq_MDH_Matrix))/(Vmr*Kia*KmB*ai+Vmr*KmB*a+Vmr*KmA*ai*b+Vmf_MDH*KmQ*ai/Keq_MDH_Matrix*p+Vmf_MDH*KmP/Keq_MDH_Matrix*q+Vmr*a*b+Vmf_MDH*KmQ/Kia/Keq_MDH_Matrix*a*p+Vmf_MDH/Keq_MDH_Matrix*p*q+Vmr*KmA/Kiq*b*q+Vmr/Kip*a*b*p+Vmf_MDH/Kib/Keq_MDH_Matrix*b*p*q);

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 1*J_MDH_Matrix ) / VWater_Matrix; % malate_Matrix
f(2,:) = ( 0  - 1*J_MDH_Matrix ) / VWater_Matrix; % NAD_Matrix
f(3,:) = ( 0  + 1*J_MDH_Matrix ) / VWater_Matrix; % oxaloacetate_Matrix
f(4,:) = ( 0  + 1*J_MDH_Matrix ) / VWater_Matrix; % NADH_Matrix
f(5,:) = ( 0  ); % [clamped] % ATP_Matrix
f(6,:) = ( 0  ); % [clamped] % ADP_Matrix
f(7,:) = ( 0  ); % [clamped] % AMP_Matrix

%% ION EQUATIONS
% COMPARTMENT Matrix:
ii = [1  2  3  4  5  6  7]; % Indices of SVs in compartment Matrix
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_Matrix*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_Matrix./Km(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_Matrix*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_Matrix./Kh(ii)+k_Matrix./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_Matrix*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_Matrix./Kh(ii)+m_Matrix./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 + 1*J_MDH_Matrix) / VWater_Matrix;
J_M = (0) / VWater_Matrix;
J_K = (0) / VWater_Matrix;
Phi_H = J_H - sum( h_Matrix*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_Matrix*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_Matrix*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_Matrix/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(8,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(9,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(10,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_MDH_Matrix];
